###
### This file will implement the unadjusted and PLEASE estimator
### for a given dataframe (df).
###
### The functions compute the risk difference and the 95%
### BCA bootstrap confidence interval.
###

### Define the required functions

# Unadjusted estimators
# Input: y (binary outcome vector) and trt (0/1 treatment indicator)
unadjusted <- function(y,trt) {
  list(mean(y[trt==1],na.rm=T),mean(y[trt==0],na.rm=T))
}

# PLEASE.calcs computes the estimated mean for a given treatment group
# This function takes the outcome (y), treatment indicator (trt),
# matrix of covariates defining the missing data model (dX),
# matrix of baseline variables defining the propensity score model (piX),
# matrix of baseline variables defining the outcome regression model (phiX),
# treatment group value (ss), and distribution of Y
PLEASE.calcs <- function(y,trt,dX,piX,phiX,ss=1,famY,ctrl,wmax){
  if(ss==1) trtnew=trt else trtnew=(1-trt)
  delta <- ifelse(is.na(y),0,1)
  if(sum(delta)==length(y)) h.wt <- rep(1,length(y))
  if(sum(delta)<length(y)) {
      hFIT <- try(glm(delta~as.matrix(cbind(trt,dX)),family=binomial,control=ctrl,na.action=na.omit),TRUE)
      failed.h <- inherits(hFIT,"try-error")
      if(failed.h) hFIT <- suppressWarnings(glm(delta~as.matrix(cbind(trt,dX)),family=binomial,control=ctrl,na.action=na.omit))
      h.wt <- hFIT$fitted           
  }
  piFIT <- try(glm(trtnew~as.matrix(piX),family=binomial,control=ctrl,na.action=na.omit),TRUE)
  failed.pi <- inherits(piFIT,"try-error")
  if(failed.pi) piFIT <- suppressWarnings(glm(trtnew~as.matrix(piX)[,1:ncol(phiX)],family=binomial,control=ctrl,na.action=na.omit))
  check.wt<-max(1/piFIT$fit[trt==ss & delta==1])
  ave.wt<-mean(trtnew/piFIT$fit)
  wt <- pmin(1/(h.wt*piFIT$fit),wmax)
  sum.wt <- sum(wt[trt==ss & delta==1])
  scaled.wt <- wt / sum.wt
  phiFIT <- try(suppressWarnings(glm(y~as.matrix(phiX),family=famY,weights=scaled.wt,subset=(trt==ss & delta==1),control=ctrl,na.action=na.omit)),TRUE)
  failed.phi <- inherits(phiFIT,"try-error")
  if(failed.phi) phiFIT <- suppressWarnings(glm(y~as.matrix(phiX),family=famY,weights=scaled.wt,subset=(trt==ss & delta==1),control=ctrl,na.action=na.omit))
  Q0 <- predict(phiFIT,as.data.frame(phiX),type="response")
  mu.hat <- mean(Q0)
  test = TRUE
  list(wt.alpha=1/piFIT$fitted,Q0=Q0,mu.hat=mu.hat,check.wt=check.wt,ave.wt=ave.wt,failed.pi=failed.pi,failed.phi=failed.phi,h.wt=h.wt)
}


# PLEASE takes the outcome (y), treatment indicator (trt), 
# baseline covariates that form the missing data model (dX),
# the propensity score model (piX), and the outcome regression
# model (phiX)
PLEASE <- function(y,trt,dX,piX,phiX,ctrl=list(epsilon=1e-7,maxit=50),wmax=20) {
  if( !all (trt %in% c(0,1) ) ) stop( "'TRT' should be binary." )
  YisBin <- all(y[!is.na(y)] %in% c(0,1))
  famY <- ifelse(YisBin,"binomial","gaussian")
  # Get the Joffe estimators and save Pr(A=1/0|W) and Pr(Y=1|A,W)
  drwls1 <- PLEASE.calcs(y,trt,dX,piX,phiX,ss=1,famY,ctrl,wmax)
  drwls0 <- PLEASE.calcs(y,trt,dX,piX,phiX,ss=0,famY,ctrl,wmax)
  # Create the new augmenting variables for the missingness model
  newd1 <- trt/(drwls1$h.wt)*(drwls1$Q0 - drwls1$mu.hat)
  newd0 <- (1-trt)/(drwls0$h.wt)*(drwls0$Q0 - drwls0$mu.hat)
  dXnew <- cbind(dX,newd1,newd0)
  # Create the new predictor for the propensity score model
  newX1 <- drwls1$Q0 - drwls1$mu.hat
  newX0 <- drwls0$Q0 - drwls0$mu.hat
  piXnew <- cbind(piX,newX1,newX0)
  # Get the updated estimate using piX and newX1
  # in the propensity score model
  trt1 <- PLEASE.calcs(y,trt,dXnew,piXnew,phiX,ss=1,famY,ctrl,wmax)
  trt0 <- PLEASE.calcs(y,trt,dXnew,piXnew,phiX,ss=0,famY,ctrl,wmax)
  mu.bar.1 <- trt1$mu.hat
  mu.bar.0 <- trt0$mu.hat
  max.wt.1 <- trt1$check.wt
  max.wt.0 <- trt0$check.wt
  ave.wt.1 <- trt1$ave.wt
  ave.wt.0 <- trt0$ave.wt
  failed.piDR.1 <- trt1$failed.pi
  failed.phiDR.1 <- trt1$failed.phi
  failed.piDR.0 <- trt0$failed.pi
  failed.phiDR.0 <- trt0$failed.phi
  list(mu.bar.1,mu.bar.0,drwls1$failed.pi,drwls1$failed.phi,drwls0$failed.pi,drwls0$failed.phi,max.wt.1,max.wt.0,ave.wt.1,ave.wt.0,failed.piDR.1,failed.phiDR.1,failed.piDR.0,failed.phiDR.0)
}

### The user should read in their data, defining the 
### dataframe as "df"

### The user should define the appropriate inputs to the functions

# Define "xs" the list of baseline variables to include in the adjusted estimator 
# Here we assume the baseline variables are the same for all working models
y <- df$y
trt <- df$trt
dX <- df[,xs]
piX <- df[,xs]
phiX <- df[,xs]
una.est <- unadjusted(y,trt)
please.est <- PLEASE(y,trt,dX,piX,phiX)

### Define the bootstrap function
my.boot <- function(d,index,xs) {
  d2 <- d[index,]
  y <- d2$y
  trt <- d2$trt
  dX <- d2[,xs]
  piX <- d2[,xs]
  phiX <- d2[,xs]
  una <- unlist(unadjusted(y,trt))[1:2]
  please <- unlist(PLEASE(y,trt,dX,piX,phiX))[1:2]
  c(una[1]-una[2],please[1]-please[2])
}

### Load required packages: you should have installed the "boot" package
library(boot)

### Define the number of bootstrap samples
k <- 1000

### Set the random seed
#set.seed(some integer)

### Run the bootstrap
boot.out <- boot(df,my.boot,R=k,xs=xs)

### Next calculate the 95% BCa confidence interval based on the bootstrap samples

## Unadjusted BCa CI
boot.ci(boot.out,index=1,type="bca")

## PLEASE BCa CI
boot.ci(boot.out,index=2,type="bca")

