* ---------------------------------------------------------------------------------------------------- ;
* macro: PLEASE                                                                                        ;
* Uses a logistic model to model missingness in the primary outcome variable, if there is any.         ;
* Uses a logistic model on two treatments to build weights.  Uses the genmod procedure to preform a    ;
* weighted regression.                                                                                 ;
*                                                                                                      ;
* Inputs:  dataset on which to perform two regressions:                                                ;
*                                                                                                      ;
*   Logistic:  miss_var ~  miss_ind_vars                                                               ;
*              where miss_var is a numeric variable taking two values 0 and 1.  The value 1 indicates  ;
*              that the outcome variable is not missing.  The variable list miss_ind_vars is passed    ;
*              to proc logistic                                                                        ;
*                                                                                                      ;
*   Logistic:  trt_var  ~  prop_ind_vars                                                               ;
*              where trt_var is a numeric variable taking two values 0 and 1, and prop_ind_vars is a   ;
*              list of numeric independent variables.  This model is passed to proc logistic.          ;
*                                                                                                      ;
*   GLM:       reg_var  ~  reg_ind_vars                                                                ;
*              where reg_var is a numeric dependent variable, and reg_ind_vars is a list of numeric    ;
*              independent variables.  These are passed to proc genmod.                                ;
*              The parameter dist gives the distribution passed to genmod, which can take the values   ;
*                  ---------------------------------------------------------------                     ;
*                  Dist =                Distribution    Link                                          ;
*                  ---------------------------------------------------------------                     ;
*                  binomial | bin | b    binomial        logit                                         ;
*                  gamma    | gam | g    gamma           inverse( power(-1))                           ;
*                  normal   | nor | n    normal          identity                                      ;
*                  poisson  | poi | p    poisson         log                                           ;
*                  ---------------------------------------------------------------                     ;
*              see the documentation for genmod for details.                                           ;
*                                                                                                      ;
* Outputs: Two output datasets are given:                                                              ;
*          The output dataset indicated by the parameter out contains:                                 ;
*               mean0        The mean corresponding to trt_var = 0                                     ;
*               mean1        The mean corresponding to trt_var = 1                                     ;
*               status       An indicator to say if all went well in the main models.  Non-zero values ;
*                            indicate a problem.                                                       ;
*                                                                                                      ;
* Note 1: For simplicity all variables should be numeric, and the full design matrix (apart from the   ;
*         intercept) should be given.  Use proc transreg or proc glmmod to encode dummy variables if   ;
*         you have a significant number of classification levels to model or encode data in parametric ;
*         splines.                                                                                     ;
* ---------------------------------------------------------------------------------------------------- ;
* uses: macro PLEASE_UTILITY                                                                           ;
* ---------------------------------------------------------------------------------------------------- ;
%macro PLEASE(
    data          =,               /* input dataset                                          */
    trt_var       =,               /* treatment variable should be 0/1 indicator variable    */
    out           = _invout,       /* output dataset                                         */
    miss_var      =.,              /* missing indicator variable (0,1=missing)               */
    miss_ind_vars =,               /* independent variable for missing data model            */
    prop_ind_vars =,               /* independent variables for propensity score model       */
    prop_maxiter  = 50,            /* maximum interation for regression                      */    
    reg_var       =,               /* dependent variable in regression                       */
    reg_ind_vars  =,               /* independent variable in regression                     */
    dist          = normal,        /* distributon for reg_var                                */
    reg_maxiter   = 50,            /* maximum iteration for regression                       */
    converge      = 1E-7,          /* convergence used by genmod (parameter change)          */
    convh         = 1E-7,          /* Hessian convergence used by genmod                     */    
    print_res     = n,             /* Print results y/n                                      */
    WtCutoff      = 20,            /* Set large weights to this value                        */
    stdWt         = 1,             /* Standardize the weights (divide Qwt by sum of Qwt )    */
    logconv       = maxiter = 50 absfconv = 1E-9 fconv = 1E-10 gconv = 1E-10 xconv = 1E-10
                                   /* convergence criteria to pass to proc logistic          */
    );

    %if "&miss_var" = "." %then %do;
        data pindata;
          set &data;
          miss_var = missing(&reg_var);
          call symput("miss_var","miss_var");
        run;
    %end;
    %else %do;
        data pindata;
          set &data;
        run;
    %end;

    * First weight on the original data;
    %PLEASE_UTILITY(  prop_data0    = pindata,
                      prop_data1    = pindata,
                      trt_var       = &trt_var,
                      miss_var      = &miss_var,
                      miss_ind_vars = &miss_ind_vars,
                      prop_ind_vars = &prop_ind_vars,
                      prop_maxiter  = &prop_maxiter,
                      reg_var       = &reg_var,
                      reg_ind_vars  = &reg_ind_vars,
                      dist          = &dist,
                      reg_maxiter   = &reg_maxiter,
                      converge      = &converge,          
                      convh         = &convh,          
                      tag           = Main_I,
                      WtCutoff      = &WtCutoff,
                      stdWt         = &stdWt,
                      logconv       = &logconv
                   );      

    * calculate new weights ;
    proc means data=_fit0 noprint;
      var fit0;
      output out=_m0 mean=m0;
    proc means data=_fit1 noprint;
      var fit1;
      output out=_m1 mean=m1; 
    data _pred_0;
      set _fit0;
      if _n_ = 1 then set _m0;
      _newx0 =  fit0 - m0 ;
      _newd0 = 0;
      if &trt_var = 0 then  _newd0 = wt * ( fit0 - m0 );
    data _pred_1;
      set _fit1;
      if _n_ = 1 then set _m1;
      _newx1 = fit1 - m1 ;
      _newd1 = 0;
      if &trt_var = 1 then  _newd1 = wt * ( fit1 - m1 );
    run;

    data _phase2;
       merge pindata
             _pred_0( keep = _newx0 _newd0 )
             _pred_1( keep = _newx1 _newd1 );
    run;
    
    * call to reweight again, this time using adding _newx variable
      to the propensity score model and _newd to the missing model; 
    %PLEASE_UTILITY(  prop_data0    = _phase2,
                      prop_data1    = _phase2,
                      trt_var       = &trt_var,
                      miss_var      = &miss_var,
                      miss_ind_vars = &miss_ind_vars _newd0 _newd1,
                      prop_ind_vars = &prop_ind_vars _newx0 _newx1, 
                      prop_maxiter  = &prop_maxiter,
                      reg_var       = &reg_var,
                      reg_ind_vars  = &reg_ind_vars,
                      dist          = &dist,
                      reg_maxiter   = &reg_maxiter,
                      converge      = &converge,          
                      convh         = &convh,          
                      tag           = Main_II,
                      WtCutoff      = &WtCutoff,
                      stdWt         = &stdWt,
                      logconv       = &logconv
                   );      

    * deal with the return codes ;
    data _fit0;
      merge _fit0 _status_main_I_0 _status_main_I_R0 _status_main_II_0 _status_main_II_R0;
      status = max(status_main_I_0, status_main_I_R0, status_main_II_0, status_main_II_R0); 
    data _fit1;
      merge _fit1 _status_main_I_1 _status_main_I_R1 _status_main_II_1 _status_main_II_R1;
      status = max(status_main_I_1, status_main_I_R1, status_main_II_1, status_main_II_R1); 

    proc means data=_fit0 noprint;
      var fit0 status;
      output out         = _main0(drop = _:)
             n(fit0)     = n0
             nmiss(fit0) = nmiss0
             mean(fit0)  = mean0
             max(status) = status0 ;
    proc means data=_fit1 noprint;
      var fit1 status;
      output out         = _main1(drop = _:)
             n(fit1)     = n1
             nmiss(fit1) = nmiss1
             mean(fit1)  = mean1
             max(status) = status1 ;
    run;

    * output dataset ;
    data &out;
      set _main0;
      set _main1;

      status = max(status0,status1);
      drop status0 status1;
   
      length status_note  $80;
      if status ne 0 then do;
        status_note = catx(" ",cats("ERR","OR:"),"-- from macro PLEASE -- one or more main models did not converge.");
        put status_note;
      end;  
      drop status_note;
      call symputx("invstatus",  put(status,8. ));  
    run;

  %if &invstatus = 0 and %upcase(&print_res) = Y %then %do;
    proc print data=&out;
    run;    
  %end;    
%mend;


* ---------------------------------------------------------------------------------------------------- ;
* macro: PLEASE_UTILITY                                                                                ;
* This is a utility macro used in the PLEASE macro.  It packages some repeditive code - building       ;
* weights and applying them in a genmod.                                                               ;
* ---------------------------------------------------------------------------------------------------- ;
%macro PLEASE_UTILITY
    (
    prop_data0    =,           /* propensity score data for treatment = 0              */
    prop_data1    =,           /* propensity score data for treatment = 1              */
    trt_var       =,           /* treatment variable assumed to take 0,1 values only   */
    miss_var      =,           /* Missing indicator variable (0,1=missing)             */
    miss_ind_vars =,           /* independent variables for missing data model         */
    prop_ind_vars =,           /* independent variables for propensity score model     */
    prop_maxiter  =,           /* maximum interation for regression                    */    
    reg_var       =,           /* dependent variable in regression                     */
    converge      = 1E-7,      /* convergence used by genmod (parameter change)        */
    convh         = 1E-7,      /* Hessian convergence used by genmod                   */    
    reg_ind_vars  =,           /* independent variable in regression                   */
    dist          = normal,    /* distributon for reg_var                              */
    reg_maxiter   = 50,        /* maximum iteration for regression                     */
    tag           =,           /* tag to distinuish different status flags             */
    WtCutoff      = 20,        /* Cut WtQ at this value if it is above it              */
    stdWt         = 0,         /* Standardize the weights -- divide Qwt by sum of QWt  */
    logconv       = maxiter = 50 absfconv = 1E-9 fconv = 1E-10 gconv = 1E-10 xconv = 1E-10
                               /* convergence criteria to pass to proc logistic        */
    );

  * ---- missing model ;
  proc means data=&prop_data0 noprint;
    var &miss_var;
    output out   = _miss0
           n     = obs0
           sum   = nmiss0    ;
  data _null_;
    set _miss0;
     call symputx("nmiss0", put(nmiss0, 8.0));
  run;
  proc means data=&prop_data1 noprint;
    var &miss_var;
    output out   = _miss1
           n     = obs1
           sum   = nmiss1;
  data _null_;
    set _miss1;
    call symputx("nmiss1", put(nmiss1, 8.0));
  run;

  %if &nmiss0 > 0 %then %do;
    proc logistic data=&prop_data0;
      model &miss_var (event="0") = &trt_var &miss_ind_vars / &logconv ;
      output out = _h0(drop= _:) p=h0;
      ods output convergencestatus=_status_miss_&tag._0( keep = status rename = ( status = status_miss_&tag._0 ));
    run;
  %end;

  %if &nmiss1 > 0 %then %do;
    proc logistic data=&prop_data1;
      model &miss_var (event="0") = &trt_var &miss_ind_vars / &logconv ;
      output out = _h1(drop= _:) p=h1;
      ods output convergencestatus=_status_miss_&tag._1( keep = status rename = ( status = status_miss_&tag._1 ));
    run;
  %end;  
    
  * ---- propensity score ;    
  proc logistic data=&prop_data0;
     model &trt_var(event="0") = &prop_ind_vars / &logconv ;
     output out = _pred0(drop= _:) p=p0;
     ods output convergencestatus=_status_&tag._0( keep = status rename = ( status = status_&tag._0 )); 
  proc logistic data=&prop_data1;
     model &trt_var(event="1") = &prop_ind_vars / &logconv ;
     output out = _pred1(drop= _:) p=p1;
     ods output convergencestatus=_status_&tag._1( keep = status rename = ( status = status_&tag._1 )); 

  %if &nmiss0 > 0 %then %do;
     data _pred0;
       merge _pred0 _h0;
       wtQ = 1 / (p0 * h0);
       wt = 1 / h0 ;
       if &trt_var = 1 then &reg_var = .;
       if &WtCutoff > . then do;
          if wtQ > &WtCutoff then WtQ = &WtCutoff;
       end;   
  %end;
  %else %do;
     data _pred0;
       set _pred0;
       wtQ = 1 / p0;
       wt = 1 ;
       if &trt_var = 1 then &reg_var = .;
       if &WtCutoff > . then do;
         if wtQ > &WtCutoff then WtQ = &WtCutoff;
       end;  
  %end;

  %if &nmiss1 > 0 %then %do;
     data _pred1;
       merge _pred1 _h1;
       wtQ = 1 / (p1 * h1);
       wt = 1 / h1 ;
       if &trt_var = 0 then &reg_var = .;
       if &WtCutoff > . then do;
         if wtQ > &WtCutoff then WtQ = &WtCutoff;
       end;  
  %end;     
  %else %do;     
     data _pred1;
       set _pred1;
       wtQ = 1 / p1;
       wt = 1 ;
       if &trt_var = 0 then &reg_var = .;
       if &WtCutoff > . then do;
         if wtQ > &WtCutoff then WtQ = &WtCutoff;
       end;  
  %end;

  %if &stdWt = 1 %then %do;
    proc means data = _pred0 noprint;
      var wtQ;
      output   out  = Qwts0( drop = _: )
               sum  = sumQwt;
    proc means data = _pred1 noprint;
      var wtQ;
      output   out  = Qwts1( drop = _: )
               sum  = sumQwt;
    data _pred0;
       set _pred0;
       if _n_ = 1 then set Qwts0;
       /* if sumQwt < 10E14 then */ wtQ = wtQ / sumQwt;
    data _pred1;
       set _pred1;
       if _n_ = 1 then set Qwts1;
       /* if sumQwt < 10E14 then */ wtQ = wtQ / sumQwt;
    run;   
  %end;

  * ----- regression model with weights ;
  proc genmod data=_pred0 descending;
     model &reg_var = &reg_ind_vars  /  dist=&dist maxiter=&reg_maxiter converge=&converge convh=&convh;
     weight wtQ;
     output out = _fit0 p = fit0;
     ods output convergencestatus = status0;
     ods output convergencestatus=_status_&tag._R0( keep = status rename = ( status = status_&tag._R0 )); 
  proc genmod data=_pred1 descending;
     model &reg_var = &reg_ind_vars  /  dist=&dist maxiter=&reg_maxiter converge=&converge convh=&convh;
     weight wtQ;
     output out = _fit1 p = fit1;
     ods output convergencestatus = status1;
     ods output convergencestatus=_status_&tag._R1( keep = status rename = ( status = status_&tag._R1 )); 
  run;
%mend;
